@@ -34,6 +34,7 @@ int main(int argc, char *argv[]) /*{{{*/
   int usage=0,o;
   const char *lp="/dev/null";
   int backslash_colon=0;
+  int do_repeat=0;
   int uppercase=0;
   int restricted=0;
   int lpfd;
@@ -44,6 +45,7 @@ int main(int argc, char *argv[]) /*{{{*/
     { "restricted", no_argument, 0, 'r' },
     { "uppercase", no_argument, 0, 'u' },
     { "backslash-colon", no_argument, 0, 'b' },
+    { "do-repeat", no_argument, 0, 'd' },
     { "version", no_argument, 0, 'V' },
 #if defined(__STDC__) && __STDC__
     { (const char*)0, 0, 0, '\0' }
@@ -60,9 +62,10 @@ int main(int argc, char *argv[]) /*{{{*/
   textdomain("bas");
 #endif
   /* parse arguments */ /*{{{*/
-  while ((o=getopt_long(argc,argv,"+bl:ruVh?",lopts,(int*)0))!=EOF) switch (o)
+  while ((o=getopt_long(argc,argv,"+bdl:ruVh?",lopts,(int*)0))!=EOF) switch (o)
   {
     case 'b': backslash_colon=1; break;
+    case 'd': do_repeat=1; break;
     case 'l': lp=optarg; break;
     case 'u': uppercase=1; break;
     case 'r': restricted=1; break;
@@ -73,8 +76,8 @@ int main(int argc, char *argv[]) /*{{{*/
   if (optind<argc) runFile=argv[optind++];
   if (usage==1)
   {
-    fputs(_("Usage: bas [-b] [-l file] [-r] [-u] [program [argument ...]]\n"),stderr);
-    fputs(_("       bas [--backslash-colon] [--lp file] [--restricted] [--uppercase] [program [argument ...]]\n"),stderr);
+    fputs(_("Usage: bas [-b] [-d] [-l file] [-r] [-u] [program [argument ...]]\n"),stderr);
+    fputs(_("       bas [--backslash-colon] [--do-repeat] [--lp file] [--restricted] [--uppercase] [program [argument ...]]\n"),stderr);
     fputs(_("       bas -h|--help\n"),stderr);
     fputs(_("       bas --version\n"),stderr);
     fputs(  "\n",stderr);
@@ -83,14 +86,15 @@ int main(int argc, char *argv[]) /*{{{*/
   }
   if (usage==2)
   {
-    fputs(_("Usage: bas [-b] [-l file] [-u] [program [argument ...]]\n"),stdout);
-    fputs(_("       bas [--backslash-colon] [--lp file] [--restricted] [--uppercase] [program [argument ...]]\n"),stdout);
+    fputs(_("Usage: bas [-b] [-d] [-l file] [-r] [-u] [program [argument ...]]\n"),stdout);
+    fputs(_("       bas [--backslash-colon] [--do-repeat] [--lp file] [--restricted] [--uppercase] [program [argument ...]]\n"),stdout);
     fputs(_("       bas -h|--help\n"),stdout);
     fputs(_("       bas --version\n"),stdout);
     fputs("\n",stdout);
     fputs(_("BASIC interpreter.\n"),stdout);
     fputs("\n",stdout);
     fputs(_("-b, --backslash-colon  convert backslashs to colons\n"),stdout);
+    fputs(_("-d, --do-repeat        convert DO to REPEAT\n"),stdout);
     fputs(_("-l, --lp               write LPRINT output to file\n"),stdout);
     fputs(_("-r, --restricted       forbid SHELL\n"),stdout);
     fputs(_("-u, --uppercase        output all tokens in uppercase\n"),stdout);
@@ -109,7 +113,7 @@ int main(int argc, char *argv[]) /*{{{*/
   bas_argv=&argv[optind];
   bas_argv0=runFile;
   /*}}}*/
-  bas_init(backslash_colon,restricted,uppercase,lpfd);
+  bas_init(backslash_colon,do_repeat,restricted,uppercase,lpfd);
   if (runFile) bas_runFile(runFile);
   else bas_interpreter();
   bas_exit();
